


#include "HShell.h"
#include "HMem.h"
#include "HMath.h"
#include "HSigP.h"
#include "HAudio.h"
#include "HWave.h"
#include "HVQ.h"
#include "HParm.h"
#include "HLabel.h"
#include "HModel.h"
#include "HUtil.h"


typedef struct {
	char hmm_name[50];
	int num_states;
	int* vec_dim;
	double** trans_mat;
	double**** mean;//
	double**** variance;//
	int num_streams;
	double*** weights;//
	int** num_mixtures;//
	double*** gconst;//
	double** s_weights;
	char cKind[5];
} sHMM;

class HMMModule
{
public:
	HMMModule (int argc, char* argv[]);
	~HMMModule ();

	char *s;
	char *hcompv_version;
	char *hcompv_vc_id;
	LabId  hmmId;
	char *hmmfn;

	MLink macroLink;             /* Link to specific HMM macro */
	HLink hmmLink;               /* Link to the physical HMM */
	HMMSet hset;                 /* HMM to be initialised with */

	char in_hmm_name[50];
	char* in_divide_name;
	char* out_hmm_name;
	char* out_dict_name;
	char* out_grammar_name;
	char* out_list_name;


	void ReportUsage(void);
	void CheckVarianceKind(void);
	void Initialise(void);
	void Initialise2(char* hmmListFn);
	void set_hmm(char* hmm_name);
	int read_hmm(sHMM* hmm);
	int print_hmm_to_file(sHMM* hmm,FILE* fid);
	int divide_hmm(int* state_numbers,int num_states,char* name,sHMM* hmm_in,sHMM* hmm_out);
	int divide_hmms(sHMM* hmm_in,sHMM* hmm_out);
	int get_num_divide();


};

HMMModule::HMMModule(int argc, char* argv[])
{
	hcompv_version = "!HVER!HCompV:   3.2.1 [CUED 15/10/03]";
	hcompv_vc_id = "$Id: HCompV.c,v 1.12 2003/10/15 08:10:13 ge204 Exp $";
	LabId  hmmId  = NULL;        /* id of model */
	hmmfn=NULL;            /* HMM definition file name */


	if(InitShell(argc,argv,hcompv_version,hcompv_vc_id)<SUCCESS)
		HError(2000,"HCompV: InitShell failed");
	InitMem();   InitLabel();
	InitMath();  InitSigP();
	InitWave();  InitAudio();
	InitVQ();    InitModel();
	if(InitParm()<SUCCESS)  
		HError(2000,"HCompV: InitParm failed");

	//if (!InfoPrinted() && NumArgs() == 0)
	//	ReportUsage();
	//if (NumArgs() == 0) Exit(0);
	//SetConfParms();

	CreateHMMSet(&hset,&gstack,FALSE);
	//while (NextArg() == SWITCHARG) 
	//{
	//	s = GetSwtArg();
	//	if (strlen(s)!=1) 
	//		HError(2019,"HCompV: Bad switch %s; must be single letter",s);
	//	switch(s[0])
	//	{
	//	case 'o':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"out_hmm_name expected");
	//		out_hmm_name = GetStrArg();
	//		break;
	//	case 'l':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"out_list_name expected");
	//		out_list_name = GetStrArg();
	//		break;
	//	case 'd':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"out_dict_name expected");
	//		out_dict_name = GetStrArg();
	//		break;
	//	case 'g':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"out_grammar_name expected");
	//		out_grammar_name = GetStrArg();
	//		break;     
	//	case 's':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"in_divide_name expected");
	//		in_divide_name = GetStrArg();
	//		break;
	//	case 'i':
	//		if (NextArg() != STRINGARG)
	//			HError(2019,"in_hmm_name expected");
	//		in_hmm_name = GetStrArg();
	//		break;
	//	default:
	//		HError(2019,"Unknown switch %s",s);
	//	}
	//}



	//Initialise();


	//FILE* temp_fid = fopen("deneme","w");

	//PrintHMMProfile(temp_fid,hmmLink);

	//SaveHMMSet(&hset,"./", "", FALSE);

	//fclose(temp_fid);
}








void HMMModule::ReportUsage(void)
{
	printf("\nUSAGE: Divide HMM [options] [hmmFile] trainFiles...\n" );
	printf(" Option                                       Default\n\n");
	printf(" -c dir  Set output directiry for CMV         none\n");
	printf(" -f f    Output vFloor as f * global var      none\n");
	printf(" -k s    spkr pattern for CMV                 none\n");
	printf(" -l s    Set segment label to s               none\n");
	printf(" -m      Update means                         off\n");
	printf(" -o fn   Store new hmm def in fn (name only)  outDir/srcfn\n");
	printf(" -q nmv  output type flags for CMV            m\n");
	printf(" -v f    Set minimum variance to f            0.0\n");
	PrintStdOpts("BCFGHILMX");
	printf("\n\n");
}


void HMMModule::CheckVarianceKind(void)
{
	int i,s,m;
	StateElem *se;
	StreamElem *ste;
	MixtureElem *me;

	for (i=2,se=hmmLink->svec+2; i < hmmLink->numStates; i++,se++)
		for (s=1,ste=se->info->pdf+1; s <= hset.swidth[0]; s++,ste++)
			for (m=1,me = ste->spdf.cpdf+1; m<=ste->nMix; m++, me++)
				if (me->mpdf->ckind == FULLC)
					HError(2028,"Full Covariance is not supported");
}


void HMMModule::Initialise2(char* hmmListFn)
{
	Boolean eSep;
	char base[MAXSTRLEN];
	char path[MAXSTRLEN];
	char ext[MAXSTRLEN];	
	hmmfn = in_hmm_name;
	
	AddMMF(&hset,in_hmm_name);
	


	//char* temp = BaseOf(hmmListFn,base);
	//temp = PathOf(hmmfn,path);

	//temp = ExtnOf(hmmfn,ext);



	//if(MakeHMMSet( &hset, BaseOf(hmmListFn,base) )<SUCCESS)
	if(MakeHMMSet( &hset, hmmListFn )<SUCCESS)
		HError(2321,"Initialise: MakeHMMSet failed");
	if(LoadHMMSet( &hset, PathOf(hmmfn,path),ExtnOf(hmmfn,ext))<SUCCESS)
		HError(2321,"Initialise: LoadHMMSet failed");
	SetParmHMMSet(&hset);



	//Boolean eSep;
	//char base[MAXSTRLEN];
	//char path[MAXSTRLEN];
	//char ext[MAXSTRLEN];

	///* Load HMM defs */     
	//if(MakeOneHMM(&hset,BaseOf(hmmfn,base))<SUCCESS)
	//	HError(2028,"Initialise: MakeOneHMM failed");

	//
	//if(LoadHMMSet(&hset,PathOf(hmmfn,path),ExtnOf(hmmfn,ext))<SUCCESS)
	//	HError(2028,"Initialise: LoadHMMSet failed");
	//SetParmHMMSet(&hset);
	//if (hset.hsKind==DISCRETEHS || hset.hsKind==TIEDHS)
	//	HError(2030,"Initialise: HCompV only uses continuous models");


	/* Get a pointer to the physical HMM */
	//hmmId = "a";
}

void HMMModule::set_hmm(char* name)
{
	Boolean eSep;
	hmmId = GetLabId(name,FALSE);
	macroLink = FindMacroName(&hset,'h',hmmId);
	if (macroLink==NULL)
		HError(2020,"Initialise: cannot find hmm %s in hset",hmmfn);
	hmmLink = (HLink)macroLink->structure;

	//CheckVarianceKind( );
	SetStreamWidths(hset.pkind,hset.vecSize,hset.swidth,&eSep);

}


void HMMModule::Initialise(void)
{
		
	hmmfn = in_hmm_name;
	Boolean eSep;
	char base[MAXSTRLEN];
	char path[MAXSTRLEN];
	char ext[MAXSTRLEN];

	/* Load HMM defs */     
	if(MakeOneHMM(&hset,BaseOf(hmmfn,base))<SUCCESS)
		HError(2028,"Initialise: MakeOneHMM failed");

	
	if(LoadHMMSet(&hset,PathOf(hmmfn,path),ExtnOf(hmmfn,ext))<SUCCESS)
		HError(2028,"Initialise: LoadHMMSet failed");
	SetParmHMMSet(&hset);
	if (hset.hsKind==DISCRETEHS || hset.hsKind==TIEDHS)
		HError(2030,"Initialise: HCompV only uses continuous models");


	/* Get a pointer to the physical HMM */
	hmmId = GetLabId(base,FALSE);
	macroLink = FindMacroName(&hset,'h',hmmId);
	if (macroLink==NULL)
		HError(2020,"Initialise: cannot find hmm %s in hset",hmmfn);
	hmmLink = (HLink)macroLink->structure;

	//CheckVarianceKind( );
	SetStreamWidths(hset.pkind,hset.vecSize,hset.swidth,&eSep);
}
//void SaveModel(char *outfn)
//{
	//if (outfn != NULL)
	//	macroLink->id = GetLabId(outfn,TRUE);
	//if(SaveHMMSet(&hset,outDir,NULL,saveBinary)<SUCCESS)
	//	HError(2011,"SaveModel: SaveHMMSet failed");
//}


int HMMModule::read_hmm(sHMM* hmm)
{

	int i,s,m,k;
	StateElem *se;
	StreamElem *ste;
	MixtureElem *me;
	MixPDF *mp;

	strcpy(hmm->cKind,"diag");
	for (i=2,se=hmmLink->svec+2; i < hmmLink->numStates; i++,se++)
		for (s=1,ste=se->info->pdf+1; s <= hset.swidth[0]; s++,ste++)
			for (m=1,me = ste->spdf.cpdf+1; m<=ste->nMix; m++, me++)
				if (me->mpdf->ckind == FULLC)
					strcpy(hmm->cKind,"full");


	strcpy(hmm->hmm_name,"null");

	hmm->num_streams = (int)hset.swidth[0];

	hmm->vec_dim = new int[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->vec_dim[i]=(int)hset.swidth[i+1];


	hmm->num_states = (int)hmmLink->numStates;


	hmm->trans_mat = new double*[hmm->num_states];
	for (i=0;i<hmm->num_states;i++)
		hmm->trans_mat[i]=new double[hmm->num_states];

	for(i = 1 ; i<=  NumRows(hmmLink->transP);i++)
	{
		for(s = 1 ; s<=  NumCols(hmmLink->transP);s++)
		{
			hmm->trans_mat[i-1][s-1] = (double)L2F(hmmLink->transP[i][s]);
		}
	}


	hmm->mean = new double***[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->mean[i] = new double**[hmm->num_states];

	hmm->variance = new double***[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->variance[i] = new double**[hmm->num_states];

	hmm->gconst = new double**[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->gconst[i] = new double*[hmm->num_states];

	hmm->weights = new double**[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->weights[i] = new double*[hmm->num_states];

	hmm->num_mixtures = new int*[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->num_mixtures[i] = new int[hmm->num_states];

	hmm->s_weights = new double*[hmm->num_streams];
	for(i=0;i<hmm->num_streams;i++)
		hmm->s_weights[i] = new double[hmm->num_states];

	for (i=2,se=hmmLink->svec+2; i < hmmLink->numStates; i++,se++)
	{
		for (s=1,ste=se->info->pdf+1; s <= hset.swidth[0]; s++,ste++)
		{	   
			hmm->num_mixtures[s-1][i-1]= ste->nMix;
			hmm->weights[s-1][i-1] = new double[ste->nMix];
			hmm->gconst[s-1][i-1] = new double[ste->nMix];
			hmm->mean[s-1][i-1] = new double*[ste->nMix];
			hmm->variance[s-1][i-1] = new double*[ste->nMix];
			hmm->s_weights[s-1][i-1] = se->info->weights[s];
			for (m=1,me = ste->spdf.cpdf+1; m<=ste->nMix; m++, me++) 
			{
				mp = me->mpdf;
				hmm->mean[s-1][i-1][m-1]=new double[hmm->vec_dim[s-1]];
				hmm->variance[s-1][i-1][m-1]=new double[hmm->vec_dim[s-1]*hmm->vec_dim[s-1]];
				hmm->weights[s-1][i-1][m-1] = me->weight;
				for (k = 1 ; k <= hmm->vec_dim[s-1] ; k++)
				{
					hmm->mean[s-1][i-1][m-1][k-1]=(double)mp->mean[k];
				}

				if (strcmp(hmm->cKind,"diag") == 0)
				{
					int a=0;
					for (k = 1 ; k <= hmm->vec_dim[s-1] ; k++)
					{
						hmm->variance[s-1][i-1][m-1][a++]=(double)mp->cov.var[k];
						for(int l=0; a<(hmm->vec_dim[s-1]*hmm->vec_dim[s-1]) && l<hmm->vec_dim[s-1];a++,l++)
							hmm->variance[s-1][i-1][m-1][a]=0;
					}
				}
				else
				{
					int a=0;
					int c=hmm->vec_dim[s-1]+2;
					int inc1=0;
					int inc2;
					for (int l=2;l<(hmm->vec_dim[s-1]+2);l++)
					{
						for(k=0;k<(l-2);k++)
							hmm->variance[s-1][i-1][m-1][a++]=0;

						if (l!=2) inc1=l;
						c+=inc1;
						inc2 = 0;
						int cc = c;
						for(int ll=l;ll<(hmm->vec_dim[s-1]+2);ll++)
						{
							if (ll!=l) inc2 = ll-1;
							cc+=inc2;
							hmm->variance[s-1][i-1][m-1][a++]=(double)mp->cov.var[cc];
						}
					}

				}

				hmm->gconst[s-1][i-1][m-1] = (double)mp->gConst;
			}
		}
	}
	return 0;
}

int HMMModule::print_hmm_to_file(sHMM* hmm,FILE* fid)
{
	//FILE* fid = fopen(name,"w");
	//fprintf(fid,"~o <VecSize> %d <USER> <StreamInfo> %d %d\n",hmm->vec_dim,hmm->num_streams,hmm->vec_dim);  
	fprintf(fid,"~h \"%s\"\n",hmm->hmm_name);
	fprintf(fid,"<BeginHMM>\n");
	fprintf(fid,"<NumStates> %d\n",hmm->num_states);
	for(int i=1;i<hmm->num_states-1;i++)
	{
		fprintf(fid,"<State> %d \n",i+1);
		fprintf(fid,"<NumMixes>");
		for(int k=0;k<hmm->num_streams;k++)
			fprintf(fid," %d",hmm->num_mixtures[k][i]);
		fprintf(fid,"\n");

		fprintf(fid,"<SWEIGHTS> %d\n",hmm->num_streams);
		for(int k=0;k<hmm->num_streams;k++)
			fprintf(fid,"%e ",hmm->s_weights[k][i]);
		fprintf(fid,"\n");

		for(int m=0;m<hmm->num_streams;m++)
		{
			fprintf(fid,"<Stream> %d\n",m+1);
			for(int k=0;k<hmm->num_mixtures[m][i];k++)
			{
				fprintf(fid,"<Mixture> %d %e\n",k+1,hmm->weights[m][i][k]);
				fprintf(fid,"<Mean> %d\n",hmm->vec_dim[m]);
				for(int j=0;j<hmm->vec_dim[m];j++)
					fprintf(fid,"%e ",hmm->mean[m][i][k][j]);    
				fprintf(fid,"\n");
				
				if ( strcmp(hmm->cKind,"full") == 0)
				{
					fprintf(fid,"<InvCovar> %d\n",hmm->vec_dim[m]);
					for(int j=0;j<hmm->vec_dim[m];j++)
					{
						for(int l=j;l<hmm->vec_dim[m];l++)
						{
							//printf("%d %e\n",j*hmm->vec_dim[m]+l,hmm->variance[m][i][k][j*hmm->vec_dim[m]+l]);
							fprintf(fid,"%e ",hmm->variance[m][i][k][j*hmm->vec_dim[m]+l]);
						}
						fprintf(fid,"\n");
					}
				}
				else
				{
					fprintf(fid,"<Variance> %d\n",hmm->vec_dim[m]);
					for(int j=0;j<(hmm->vec_dim[m]*hmm->vec_dim[m]);j++)
					{
						fprintf(fid,"%e ",hmm->variance[m][i][k][j]);
						j+=hmm->vec_dim[m];
					}
					fprintf(fid,"\n");
				}

				fprintf(fid,"<GCONST> %e\n",hmm->gconst[m][i][k]);
			}
		}
	}
	fprintf(fid,"<TransP> %d\n",hmm->num_states);
	for(int i=0;i<hmm->num_states;i++)
	{
		for(int j=0;j<hmm->num_states;j++)
		{
			fprintf(fid,"%e ",hmm->trans_mat[i][j]);
		}
		fprintf(fid,"\n");
	}
	fprintf(fid,"<EndHMM>\n");   
	//fclose(fid);
	return 0;
}


int HMMModule::divide_hmm(int* state_numbers,int num_states,char* name,sHMM* hmm_in,sHMM* hmm_out)
{

	strcpy(hmm_out->cKind,hmm_in->cKind);
	strcpy(hmm_out->hmm_name,name);
	hmm_out->num_states=num_states+2;
	hmm_out->num_streams=hmm_in->num_streams;
	hmm_out->vec_dim = new int[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->vec_dim[i]=hmm_in->vec_dim[i];

	hmm_out->trans_mat = new double*[hmm_out->num_states];
	for (int i=0;i<hmm_out->num_states;i++)
		hmm_out->trans_mat[i]=new double[hmm_out->num_states];


	hmm_out->mean = new double***[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->mean[i] = new double**[hmm_out->num_states];

	hmm_out->variance = new double***[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->variance[i] = new double**[hmm_out->num_states];

	hmm_out->gconst = new double**[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->gconst[i] = new double*[hmm_out->num_states];


	hmm_out->weights = new double**[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->weights[i] = new double*[hmm_out->num_states];

	hmm_out->num_mixtures = new int*[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->num_mixtures[i] = new int[hmm_out->num_states];

	hmm_out->s_weights = new double*[hmm_out->num_streams];
	for(int i=0;i<hmm_out->num_streams;i++)
		hmm_out->s_weights[i] = new double[hmm_out->num_states];



	for(int i=1;i<hmm_out->num_states-1;i++)
	{

		for(int m=0;m<hmm_out->num_streams;m++)
		{

			hmm_out->mean[m][i] = new double*[hmm_in->num_mixtures[m][state_numbers[i-1]-1]];
			hmm_out->variance[m][i] = new double*[hmm_in->num_mixtures[m][state_numbers[i-1]-1]];
			hmm_out->gconst[m][i] = new double[hmm_in->num_mixtures[m][state_numbers[i-1]-1]];
			hmm_out->weights[m][i] = new double[hmm_in->num_mixtures[m][state_numbers[i-1]-1]];

			hmm_out->num_mixtures[m][i]=hmm_in->num_mixtures[m][state_numbers[i-1]-1];

			hmm_out->s_weights[m][i]=hmm_in->s_weights[m][state_numbers[i-1]-1];


			for(int k=0;k<hmm_out->num_mixtures[m][i];k++)
			{


				hmm_out->mean[m][i][k]=new double[hmm_out->vec_dim[m]];

				hmm_out->variance[m][i][k]=new double[hmm_out->vec_dim[m]*hmm_out->vec_dim[m]];


				for(int j=0;j<hmm_out->vec_dim[m];j++)
				{
					hmm_out->mean[m][i][k][j]=hmm_in->mean[m][state_numbers[i-1]-1][k][j];
					hmm_out->variance[m][i][k][j]=hmm_in->variance[m][state_numbers[i-1]-1][k][j];
				}

				//int k=state_numbers[i-1];

				hmm_out->weights[m][i][k]=hmm_in->weights[m][state_numbers[i-1]-1][k];
				hmm_out->gconst[m][i][k]=hmm_in->gconst[m][state_numbers[i-1]-1][k];
			}
		}
	}

	for(int i=0;i<hmm_out->num_states;i++)
	{
		for(int j=0;j<hmm_out->num_states;j++)
		{
			hmm_out->trans_mat[i][j]=0.0;		
		}
	}

	hmm_out->trans_mat[0][1]=1.0;


	for(int i=1;i<hmm_out->num_states-1;i++)
	{
		for(int j=1;j<hmm_out->num_states-1;j++)
		{
			hmm_out->trans_mat[i][j]=hmm_in->trans_mat[state_numbers[i-1]-1][state_numbers[j-1]-1];

			//printf("%d,%d,%f\n",state_numbers[i-1]-1,state_numbers[j-1]-1,hmm_in->trans_mat[state_numbers[i-1]-1][state_numbers[j-1]-1]);
		}
	}


	hmm_out->trans_mat[hmm_out->num_states-2][hmm_out->num_states-1]=
		1.0-hmm_out->trans_mat[hmm_out->num_states-2][hmm_out->num_states-2];



	return 0;
}
int HMMModule::divide_hmms(sHMM* hmm_in,sHMM* hmm_out)
{
	FILE* fid = fopen (in_divide_name,"r");

	FILE* fid_out = fopen(out_hmm_name,"w");

	fprintf(fid_out,"~o\n");
	fprintf(fid_out,"<StreamInfo> %d",hmm_in->num_streams);
	int vecsize = 0;

	for(int i=0;i<hmm_in->num_streams;i++)
	{
		fprintf(fid_out," %d",hmm_in->vec_dim[i]);
		vecsize = vecsize + hmm_in->vec_dim[i];
	}

	if (strcmp(hmm_in->cKind,"diag") == 0)
		fprintf(fid_out,"\n<VecSize> %d <NULLD><USER><DIAGC>\n",vecsize);
	else
		fprintf(fid_out,"\n<VecSize> %d <NULLD><USER><FULLC>\n",vecsize);




	int num_to_divide;
	fscanf(fid,"%d",&num_to_divide);

	//hmm_out = new sHMM[num_to_divide];

	for (int i=0;i<num_to_divide;i++)
	{
		char name[50];
		//char out_name[255];
		int num_states;
		fscanf(fid,"%s",name);
		fscanf(fid,"%d",&num_states);
		//strcpy(out_name,argv[3]);
		//strcat(out_name,name);
		int* state_numbers = new int[num_states];
		for(int j=0;j<num_states;j++)
		{
			fscanf(fid,"%d",&state_numbers[j]);
			//printf("%d,",state_numbers[j]);
		}
		//printf("\n");
		divide_hmm(state_numbers,num_states,name,hmm_in,&hmm_out[i]);
		print_hmm_to_file(&hmm_out[i],fid_out);
		//print_hmm_to_file(&hmm_out[i],"a");

				



		delete []state_numbers;	
	}
	fclose(fid);
	fclose(fid_out);
	fid = fopen(out_grammar_name,"w");
	fprintf(fid,"(< ( ");
	for (int i=0;i<num_to_divide;i++)
	{
		if(i!=num_to_divide-1)
			fprintf(fid,"%s | ",hmm_out[i].hmm_name);
		else
			fprintf(fid,"%s ) >)",hmm_out[i].hmm_name);
	}
	fclose(fid);
	fid = fopen(out_dict_name,"w");
	for (int i=0;i<num_to_divide;i++)
		fprintf(fid,"%s %s\n",hmm_out[i].hmm_name,hmm_out[i].hmm_name);
	fclose(fid);
	fid = fopen(out_list_name,"w");
	for (int i=0;i<num_to_divide;i++)
		fprintf(fid,"%s\n",hmm_out[i].hmm_name);
	fclose(fid);

	return(num_to_divide);


}

HMMModule::~HMMModule ()
{


}
int HMMModule::get_num_divide()
{

	int num_to_divide;
	FILE* fid = fopen (in_divide_name,"r");
	fscanf(fid,"%d",&num_to_divide);
	fclose(fid);
	return num_to_divide;
}

