#ifndef LINKED_LIST_H
#define LINKED_LIST_H
#include <iostream>


//////////////////////////////////////////////////////
///////////////Node Decleration///////////////////////
//////////////////////////////////////////////////////
template <class T>
class Node
{
    private:
		// next is the address of the following node
        Node<T> *next;
    public:
    	// the data is public
        T data1;
		T data2;
		T data3;
		T data4;
		T data5;
    
		// constructor
        Node (const T& item1, const T& item2, const T& item3, const T& item4, const T& item5, Node<T>* ptrnext = NULL);
        
        // list modification methods
        void InsertAfter(Node<T> *p);
        Node<T> *DeleteAfter(void);
        
        // obtain the address of the next node
        Node<T> *NextNode(void) const;
};

//////////////////////////////////////////////////////
///////////////Linked List Decleration////////////////
//////////////////////////////////////////////////////
template <class T>
class LinkedList
{
   private:
      // pointers maintain access to front and rear of list
      Node<T> *front, *rear;
      
      /* used for data retrieval, insertion and deletion, list traversal*/
      Node<T> *prevPtr, *currPtr;
      
      // number of elements in the list
      int size;
      
      // position in list. used by Reset method
      int position;
      // private methods to allocate and deallocate nodes
      Node<T> *GetNode(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5, Node<T> *ptrNext=NULL);
      void FreeNode(Node<T> *p);
      
      // copies list L to current list
      void CopyList(const LinkedList<T>& L);
      
   public:
      // constructors
      LinkedList(void);
      LinkedList(const LinkedList<T>& L);
      
      // destructor
      ~LinkedList(void);
// assignment operator
      LinkedList<T>& operator= (const LinkedList<T>& L);
      
// methods to check list status
      int ListSize(void) const;               
      int ListEmpty(void) const;
      
      // Traversal methods
      void Reset(int pos = 0);
	  void Next(void);
      int EndOfList(void) const;
      int CurrentPosition(void) const;
      
      // Insertion methods
      void InsertFront(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5);
      void InsertRear(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5);
      void InsertAt(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5);
      void InsertAfter(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5);
      
      // Deletion methods
      T DeleteFront(void);
      Node<T>* DeleteAt(void);

	  // Data retrieval/modification
	  T& Data5(void);
	  T& Data4(void);
      T& Data3(void);
	  T& Data2(void);
	  T& Data1(void);

      // method to clear the list
      void ClearList(void);
/*      
      // this class (Ch. 12) needs access to front
      friend class SeqListIterator<T>;
*/
};
///////////////////////////////////////////////////////////
/////////////////Node Class Implementation/////////////////
///////////////////////////////////////////////////////////

//constructor
template <class T>
Node <T>::Node(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5,  Node<T>* ptrnext): data1(item1), data2(item2), data3(item3), data4(item4), data5(item5), next(ptrnext)
{}

// return value of private member next
template <class T>
Node<T> *Node<T>::NextNode(void) const
{
    return next;
} 

// insert a node p after the current one
template <class T>
void Node<T>::InsertAfter(Node<T> *p)
{
	// p points to successor of the current node, 
	// and current node  points to p.
    p->next = next;
    next = p;
}

// delete the node following current and return its address
template <class T>
Node<T> *Node<T>::DeleteAfter(void)
{
	// save address of node to be deleted
    Node<T> *tempPtr = next;

	// if there isn't a successor, return NULL
    if (next == NULL)
        return NULL;
        
	// current node points to successor of tempPtr.
    next = tempPtr->next;
    
	// return the pointer to the unlinked node
    return tempPtr;
}


///////////////////////////////////////////////////////////
//////////Linked List Class Implementation/////////////////
///////////////////////////////////////////////////////////

// create empty list by setting pointers to NULL, size to 0
// and list position to -1
template <class T> //Constructor
LinkedList<T>::LinkedList(void): front(NULL), rear(NULL),
      prevPtr(NULL),currPtr(NULL), size(0), position(-1)
{}

template <class T> //Copy Constructor
LinkedList<T>::LinkedList(const LinkedList<T>& L)
{
   front = rear = NULL;
   prevPtr = currPtr = NULL;
   size = 0;
   position = -1;
   CopyList(L);
}

template <class T> //Destructor
LinkedList<T>::~LinkedList(void)
{
   ClearList();
}

template <class T>
Node<T> *LinkedList<T>::GetNode(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5, Node<T>* ptrNext)
{
   Node<T> *p;

   p = new Node<T>(item1,item2,item3,item4,item5,ptrNext);
   if (p == NULL)
   {
      //cout << "Memory allocation failure!\n";
      exit(1);
   }
   return p;
}

template <class T>
void LinkedList<T>::FreeNode(Node<T> *p)
{
   delete p;
}

// copy L to the current list, which is assumed to be empty
template <class T>
void LinkedList<T>::CopyList(const LinkedList<T>& L)
{
   // use p to chain through L
   Node<T> *p = L.front;
   int pos;

   // insert each element in L at the rear of current object
   while (p != NULL)
   {
      InsertRear(p->data1, p->data2, p->data3, p->data4, p->data5);
      p = p->NextNode();
   }
   
   // if list is empty return
   if (position == -1)
      return;

   // reset prevPtr and currPtr in the new list
   prevPtr = NULL;
   currPtr = front;
   for (pos = 0; pos != position; pos++)
   {
      prevPtr = currPtr;
      currPtr = currPtr->NextNode();
   }
}


template <class T>
void LinkedList<T>::ClearList(void)
{
   Node<T> *currPosition, *nextPosition;

   currPosition = front;
   while(currPosition != NULL)
   {
      // get address of next node and delete current node
      nextPosition = currPosition->NextNode();
      FreeNode(currPosition);
      currPosition = nextPosition;  // Move to next node
   }
   front = rear = NULL;
   prevPtr = currPtr = NULL;
   size = 0;
   position = -1;
}


// reset the list position to pos
template <class T>
void LinkedList<T>::Reset(int pos)
{
   int startPos;
   
   // if the list is empty, return
   if (front == NULL)
      return;
      
   // if the position is invalid, terminate the program
   if (pos < 0 || pos > size-1)
   {
      //cerr << "Reset: Invalid list position: " << pos 
      //     << endl;
      return;
   }
   
   // move list traversal mechanism to node pos
   if(pos == 0)
   {
      // reset to front of the list
      prevPtr = NULL;
      currPtr = front;
      position = 0;
   }
   else
   // reset currPtr, prevPtr, and position
   {
       currPtr = front->NextNode();
       prevPtr = front;
       startPos = 1;
	   // move right until position == pos 
	   for(position=startPos; position != pos; position++)
	   {
	       // move both traversal pointers forward
	       prevPtr = currPtr;
	       currPtr = currPtr->NextNode();
      }
   }
}

// Insert item after the current list position

template <class T>
void LinkedList<T>::InsertAfter(const T& item1, const T& item2, const T& item3, const T& item4, const T& item5)
{
   Node<T> *p;

   p = GetNode(item1, item2, item3, item4, item5);
   if (front == NULL)       // inserting into an empty list
   {
      front = currPtr = rear = p;
      position = 0;
   }
   else
   {
      // inserting after last node of list
      if (currPtr == NULL)
      currPtr = prevPtr;
      currPtr->InsertAfter(p);
      if (currPtr == rear)
      {
        rear = p;
        position = size;
      }
      else
      	position++;
      prevPtr = currPtr;
      currPtr = p;
   }
   size++;              // increment list size
}

// Delete the node at the current list position     
template <class T>
Node<T>* LinkedList<T>::DeleteAt(void)
{
   Node<T> *p;

   // error if empty list or at end of list
   if (currPtr == NULL)
   {
      cerr << "Invalid deletion!" << endl;
      exit(1);
   }
   
   // deletion must occur at front node or inside the list
   if (prevPtr == NULL)
   {
      // save address of front and unlink it. if this
      // is the last node, front becomes NULL
      p = front;
      front = front->NextNode();
   }
	else
      // unlink interior node after prevPtr. save address
      p = prevPtr->DeleteAfter();
      
   // if rear is deleted, new rear is prevPtr and position
   // is decremented; otherwise, position is the same
   // if p was last node, rear = NULL and position = -1
   if (p == rear)
   {
      rear = prevPtr;
      position--;
   }
        
   // move currPtr past deleted node. if p is last node 
   // in the list, currPtr becomes NULL
   currPtr = p->NextNode();
   
   // free the node and decrement the list size
   //FreeNode(p);
   size--;
   return p;
   
}
template <class T>
int LinkedList<T>::ListSize(void) const
{
	return size;
}

template <class T>
int LinkedList<T>::ListEmpty(void) const
{
	return size == 0;
}

template <class T>
void LinkedList<T>::Next(void)
{
	if(currPtr == NULL) // we are the end of list or list is empty
	{
//		cout<<"ERROR! we are the end of list or list is empty!"<<endl;
//		exit(1);
	}
	prevPtr = currPtr;
	currPtr = currPtr->NextNode();
	position++;
}

template <class T>
int LinkedList<T>::EndOfList(void) const
{
	return (currPtr == NULL);
}

template <class T>
T& LinkedList<T>::Data1(void)
{
	return currPtr->data1;
}

template <class T>
T& LinkedList<T>::Data2(void)
{
	return currPtr->data2;
}

template <class T>
T& LinkedList<T>::Data3(void)
{
	return currPtr->data3;
}

template <class T>
T& LinkedList<T>::Data4(void)
{
	return currPtr->data4;
}

template <class T>
T& LinkedList<T>::Data5(void)
{
	return currPtr->data5;
}

template <class T>
int LinkedList<T>::CurrentPosition(void) const
{
	return position;
}

#endif